@@ -13,10 +13,11 @@
 # limitations under the License.
 
 from .circuit import Circuit, BlueqatGlobalSetting
+from .decorators import circuitmacro
 from . import pauli
 from . import utils
 from . import vqe
 from . import wq
 from ._version import __version__
 
-__all__ = ["pauli", "utils", "vqe", "wq", "Circuit", "BlueqatGlobalSetting"]
+__all__ = ["pauli", "utils", "vqe", "wq", "circuitmacro", "Circuit", "BlueqatGlobalSetting"]
