@@ -194,17 +194,14 @@ def _rxgate(qubits: np.ndarray, n_qubits: _QSIdx, target: _QSIdx,
       nogil=True,
       parallel=True,
       fastmath=FASTMATH)
-def _u3gate(qubits: np.ndarray, n_qubits: _QSIdx, target: _QSIdx,
-            theta: np.float64, phi: np.float64, lambd: np.float64) -> None:
-    theta *= 0.5
-    cos = math.cos(theta)
-    sin = math.sin(theta)
-    expadd = cmath.exp((phi + lambd) * 0.5j)
-    expsub = cmath.exp((phi - lambd) * 0.5j)
-    a = expadd.conjugate() * cos
-    b = -expsub.conjugate() * sin
-    c = expsub * sin
-    d = expadd * cos
+def _ugate(qubits: np.ndarray, n_qubits: _QSIdx, target: _QSIdx,
+        theta: np.float64, phi: np.float64, lam: np.float64, gamma: np.float64) -> None:
+    cos = math.cos(theta * 0.5)
+    sin = math.sin(theta * 0.5)
+    a = cos * cmath.exp(1j * gamma)
+    b = -sin * cmath.exp(1j * (gamma + lam))
+    c = sin * cmath.exp(1j * (gamma + phi))
+    d = cos * cmath.exp(1j * (gamma + phi + lam))
     lower_mask = (1 << _QSMask(target)) - 1
     for i in prange(1 << (_QSMask(n_qubits) - 1)):
         i0 = _shifted(lower_mask, i)
@@ -340,6 +337,30 @@ def _cphasegate(qubits: np.ndarray, n_qubits: _QSIdx,
         qubits[i11] *= eit
 
 
+@njit(nogil=True, parallel=True, fastmath=FASTMATH)
+def _cugate(qubits: np.ndarray, n_qubits: _QSIdx, controls_target: np.ndarray,
+        theta: np.float64, phi: np.float64, lam: np.float64, gamma: np.float64) -> None:
+    cos = math.cos(theta * 0.5)
+    sin = math.sin(theta * 0.5)
+    m11 = cos * cmath.exp(1j * gamma)
+    m12 = -sin * cmath.exp(1j * (gamma + lam))
+    m21 = sin * cmath.exp(1j * (gamma + phi))
+    m22 = cos * cmath.exp(1j * (gamma + phi + lam))
+    c_mask = _QSMask(0)
+    for c in controls_target[:-1]:
+        c_mask |= _QSMask(1) << c
+    t_mask = 1 << controls_target[-1]
+    n_loop = 1 << (_QSMask(n_qubits) - _QSMask(len(controls_target)))
+    masks = _create_masks(controls_target)
+    for i in prange(n_loop):
+        i10 = _mult_shifted(masks, i) | c_mask
+        i11 = i10 | t_mask
+        t = qubits[i10]
+        u = qubits[i11]
+        qubits[i10] = m11 * t + m12 * u
+        qubits[i11] = m21 * t + m22 * u
+
+
 @njit(locals={'lower_mask': _QSMask},
       nogil=True,
       parallel=True,
@@ -621,14 +642,13 @@ class NumbaBackend(Backend):
         return ctx
 
     @staticmethod
-    def gate_cz(gate: CZGate,
-                ctx: _NumbaBackendContext) -> _NumbaBackendContext:
-        """Implementation of CZ gate."""
+    def gate_cu(gate: CUGate, ctx: _NumbaBackendContext) -> _NumbaBackendContext:
+        """Implementation of CU gate."""
         qubits = ctx.qubits
         n_qubits = ctx.n_qubits
         for control, target in gate.control_target_iter(n_qubits):
-            _czgate(qubits, n_qubits,
-                    np.ndarray([control, target], dtype=_QBIdx_dtype))
+            _cugate(qubits, n_qubits, np.array([control, target], dtype=_QBIdx_dtype),
+                gate.theta, gate.phi, gate.lam, gate.gamma)
         return ctx
 
     @staticmethod
@@ -642,6 +662,17 @@ class NumbaBackend(Backend):
                     np.array([control, target], dtype=_QBIdx_dtype))
         return ctx
 
+    @staticmethod
+    def gate_cz(gate: CZGate,
+                ctx: _NumbaBackendContext) -> _NumbaBackendContext:
+        """Implementation of CZ gate."""
+        qubits = ctx.qubits
+        n_qubits = ctx.n_qubits
+        for control, target in gate.control_target_iter(n_qubits):
+            _czgate(qubits, n_qubits,
+                    np.ndarray([control, target], dtype=_QBIdx_dtype))
+        return ctx
+
     @staticmethod
     def gate_rx(gate: RXGate,
                 ctx: _NumbaBackendContext) -> _NumbaBackendContext:
@@ -754,27 +785,13 @@ class NumbaBackend(Backend):
         return ctx
 
     @staticmethod
-    def gate_u1(gate: U1Gate,
+    def gate_u(gate: UGate,
                 ctx: _NumbaBackendContext) -> _NumbaBackendContext:
-        """Implementation of U1 gate."""
+        """Implementation of U gate."""
         qubits = ctx.qubits
         n_qubits = ctx.n_qubits
-        angle = gate.lambd
         for target in gate.target_iter(n_qubits):
-            _rzgate(qubits, n_qubits, target, angle)
-        return ctx
-
-    @staticmethod
-    def gate_u3(gate: U3Gate,
-                ctx: _NumbaBackendContext) -> _NumbaBackendContext:
-        """Implementation of U3 gate."""
-        qubits = ctx.qubits
-        n_qubits = ctx.n_qubits
-        theta = gate.theta
-        phi = gate.phi
-        lambd = gate.lambd
-        for target in gate.target_iter(n_qubits):
-            _u3gate(qubits, n_qubits, target, theta, phi, lambd)
+            _ugate(qubits, n_qubits, target, gate.theta, gate.phi, gate.lam, gate.gamma)
         return ctx
 
     @staticmethod
