@@ -1,8 +1,31 @@
 from collections import Counter
-from .backendbase import Backend
-import blueqat.gate as bqgate
+import math
+from typing import Any, List, Optional
+
 import numpy as np
 
+from .backendbase import Backend
+import blueqat.gate as bqgate
+
+class _U3(bqgate.OneQubitGate):
+    lowername = '_u3'
+    def __init__(self, targets, theta, phi, lam):
+        super().__init__(targets, (theta, phi, lam))
+
+class _CU3(bqgate.TwoQubitGate):
+    lowername = '_cu3'
+    def __init__(self, targets, theta, phi, lam):
+        super().__init__(targets, (theta, phi, lam))
+
+class _Expii(bqgate.OneQubitGate):
+    lowername = '_expii'
+    def __init__(self, targets, theta):
+        super().__init__(targets, (theta,))
+
+class _CExpii(bqgate.TwoQubitGate):
+    lowername = '_cexpii'
+    def __init__(self, targets, theta):
+        super().__init__(targets, (theta,))
 
 class QgateBackend(Backend):
     def __init__(self):
@@ -40,14 +63,14 @@ class QgateBackend(Backend):
                 'crz': (gtype.RZ, 1, 1),
                 'cphase': (gtype.U1, 1, 1),
                 # U gate
-                'u1': (gtype.RZ, 0, 1),
-                'u2': (gtype.U2, 0, 1),
-                'u3': (gtype.U, 0, 1),
-                # gtype of U1 gate is RZ instead of U1, for compatibility with OpenQASM.
+                'u': None,
+                '_u3': (gtype.U, 0, 1),
                 # controlled U gate
-                'cu1': (gtype.U1, 1, 1),
-                'cu2': (gtype.U2, 1, 1),
-                'cu3': (gtype.U, 1, 1),
+                'cu': None,
+                '_cu3': (gtype.U, 1, 1),
+                # global phase
+                '_expii': (gtype.ExpiI, 0, 1),
+                '_cexpii': (gtype.ExpiI, 1, 1),
                 # swap and multi-controlled-bit gate
                 'swap': (gtype.SWAP, 0, 2),
                 'ccx': (gtype.X, 2, 1),
@@ -56,7 +79,8 @@ class QgateBackend(Backend):
 
             if hasattr(
                     qgate,
-                    '__version__') and qgate.__version__.startswith('0.4.0'):
+                    '__version__') and qgate.__version__.startswith('0.4.'):
+                this.version = qgate.__version__
                 # register new gate types in Qgate 0.4
                 QgateBackend.gatetypes['rxx'] = (gtype.RXX, 0, 2)
                 QgateBackend.gatetypes['ryy'] = (gtype.RYY, 0, 2)
@@ -99,6 +123,7 @@ class QgateBackend(Backend):
                 QgateBackend.create_simulator = create_simulator_04
                 QgateBackend.create_sampler = create_sampler_04
             else:
+                this.version = ''
                 # wrappers to absorb interface changes for Qgate 0.3 or older
                 def set_qregs_03(gate, qreg, controls):
                     gate.set_qreg(qreg)
@@ -146,30 +171,30 @@ class QgateBackend(Backend):
                 QgateBackend.create_simulator = create_simulator_03
                 QgateBackend.create_sampler = create_sampler_03
 
-    def run(self, gates, n_qubits, *args, **kwargs):
+    def run(self,
+            gates: List[bqgate.Operation],
+            n_qubits: int,
+            returns: Optional[str] = None,
+            shots: Optional[int] = None,
+            sampling: str = 'qgate',
+            initial: Optional[np.ndarray] = None,
+            **kwargs) -> Any:
         self.n_qubits = n_qubits
-        # returns
-        r = kwargs.get('returns', '')
-        sampling = kwargs.get('sampling', 'qgate')
-        shots = kwargs.get('shots', 0)
+        r = returns or ''
+        shots = shots or 0
         if shots != 0 and r == '':
             r = 'shots'
         elif r == '':
             r = 'statevector'
 
-        if 'returns' in kwargs:
-            del kwargs['returns']
-        if 'sampling' in kwargs:
-            del kwargs['sampling']
-        if 'shots' in kwargs:
-            del kwargs['shots']
-
         sim = QgateBackend.create_simulator(kwargs)
 
         # creating registers and references.
         self.qregs = [model.Qreg() for _ in range(n_qubits)]
         self.refs = [model.Reference() for _ in range(n_qubits)]
         sim.qubits.set_ordering(self.qregs)
+        if initial is not None:
+            sim.qubits.add_qreg_group(self.qregs, initial)
 
         ops = self.convert_operators(gates)
         # qgate.dump(ops)
@@ -184,7 +209,7 @@ class QgateBackend(Backend):
             vec, shots = self.get_state_vector_and_sample(sim, ops)
             return vec, Counter(shots)
         else:
-            raise RuntimeError('Unkown returns token, {}.'.format(r))
+            raise RuntimeError('Unknown returns token, {}.'.format(r))
 
     def convert_operators(self, gates):
         ops = list()
@@ -195,6 +220,16 @@ class QgateBackend(Backend):
                     ops += self.convert_one_qubit_gate(op, typeinfo)
                 else:
                     ops += self.convert_multi_qubit_gate(op, typeinfo)
+            elif op.lowername == 'u':
+                ops += self.convert_operators([
+                    _U3(op.targets, op.theta, op.phi, op.lam),
+                    _Expii(op.targets, op.gamma + 0.5 * (op.phi + op.lam))
+                ])
+            elif op.lowername == 'cu':
+                ops += self.convert_operators([
+                    _CU3(op.targets, op.theta, op.phi, op.lam),
+                    _CExpii(op.targets, op.gamma + 0.5 * (op.phi + op.lam))
+                ])
             elif op.lowername in ('measure', 'm'):
                 # "measure": gate.Measurement, "m": gate.Measurement,
                 ops += self.convert_measure(op)
@@ -245,7 +280,8 @@ class QgateBackend(Backend):
         ctrl_qregs = [self.qregs[idx] for idx in ctrls]
         targets = gate.targets[n_ctrls:]
         target_qregs = [self.qregs[idx] for idx in targets]
-        g = QgateBackend.create_multi_qubit_gate(gate, typeinfo, ctrl_qregs, target_qregs)
+        g = QgateBackend.create_multi_qubit_gate(gate, typeinfo, ctrl_qregs,
+                                                 target_qregs)
         glist.append(g)
         return glist
 
