@@ -13,13 +13,18 @@
 # limitations under the License.
 """Utilities for convenient."""
 from collections import Counter
-from typing import Union, Tuple
+import typing
+from typing import Dict, Tuple, Union
 
 import numpy as np
 
+if typing.TYPE_CHECKING:
+    from . import Circuit
+
 
 def to_inttuple(
-        bitstr: Union[str, Counter, dict]) -> Union[Tuple[int], Counter, dict]:
+    bitstr: Union[str, Counter, Dict[str, int]]
+) -> Union[Tuple, Counter, Dict[Tuple, int]]:
     """Convert from bit string likes '01011' to int tuple likes (0, 1, 0, 1, 1)
 
     Args:
@@ -69,3 +74,26 @@ def check_unitarity(mat: np.ndarray) -> bool:
         # Not a square matrix.
         return False
     return np.allclose(mat @ mat.T.conjugate(), np.eye(shape[0]))
+
+
+def circuit_to_unitary(circ: 'Circuit', *runargs, **runkwargs):
+    """Make circuit to unitary. This function is experimental feature and
+    may changed or deleted in the future."""
+
+    # To avoid circuilar import, import here.
+    from . import Circuit
+
+    runkwargs.setdefault('returns', 'statevector')
+    runkwargs.setdefault('ignore_global', False)
+    n_qubits = circ.n_qubits
+    vecs = []
+    if n_qubits == 0:
+        return np.array([[1]])
+    for i in range(1 << n_qubits):
+        bitmask = tuple(k for k in range(n_qubits) if (1 << k) & i)
+        c = Circuit()
+        if bitmask:
+            c.x[bitmask]
+        c += circ
+        vecs.append(c.run(*runargs, **runkwargs))
+    return np.array(vecs).T
