@@ -424,10 +424,11 @@ class Opt:
 
     def qi(self):
         nn = len(self.qubo)
+        qubo = np.triu(self.qubo + np.transpose(self.qubo)) - np.eye(nn) * self.qubo
         self.J = [np.random.choice([1., 1.], nn) for j in range(nn)]
         for i in range(nn):
             for j in range(i + 1, nn):
-                self.J[i][j] = self.qubo[i][j] / 4
+                self.J[i][j] = qubo[i][j] / 4
 
         self.J = np.triu(self.J) + np.triu(self.J, k=1).T
 
@@ -435,7 +436,7 @@ class Opt:
             sum = 0
             for j in range(nn):
                 if i == j:
-                    sum += self.qubo[i][i] * 0.5
+                    sum += qubo[i][i] * 0.5
                 else:
                     sum += self.J[i][j]
             self.J[i][i] = sum
@@ -546,7 +547,7 @@ class Opt:
             self.E.append(EE)
             qtemp = (np.asarray(q, int) + 1) / 2
             qq.append([int(s) for s in qtemp])
-            if verbose == True:
+            if verbose:
                 print(i, ':', [int(s) for s in qtemp])
             if shots == 1:
                 qq = qq[0]
@@ -665,6 +666,6 @@ class Opt:
             model_qubo = self.rbm_model_qubo(shots=shots, targetT=targetT)
             self.qubo = np.asarray(self.qubo) * alpha + epsilon * (
                 np.asarray(data_qubo) - np.asarray(model_qubo))
-            if verbose == True:
+            if verbose:
                 print("epoch", i, ":", self.qubo)
 
